use warnings;
use strict;

use MooseX::Attribute::ENV;
use Test::More tests => 1;

diag( "Testing MooseX::Attribute::ENV $MooseX::Attribute::ENV::VERSION, Perl $], $^X" );
use_ok( 'MooseX::Attribute::ENV' );


